using System;
using System.Data;
using System.Data.SqlClient;

namespace DataBindingSample
{
	/// <summary>
	/// Summary description for SimpleBusObj.
	/// </summary>
	public class SimpleBusObj
	{
		
		protected SqlConnection Conn;
		
		/// <summary>
		/// Public external reference to the DataSet for this bus object
		/// </summary>
    	public DataSet DataSet;

		/// <summary>
		/// Public external reference of the DataRow that holds the result
		/// from Load() or like operations. Table name for this operation
		/// should always be this.TableName + "_Record" pointing at row 0
		/// </summary>
		public DataRow DataRow;


		public string ConnectionString;
		public string TableName = "";
		public string PkField = "Pk";
		public string ErrorMsg = "";
		public bool Error = false;

		
		public  SimpleBusObj()
		{
			//
			// TODO: Add constructor logic here
			//
		}

		/// *********************
		/// DATA UTILITY ROUTINES	
		/// *********************
		/// these are not exposed throught the Web Service
		/// and really belong into an external class.	


		/// <summary>
		/// Sets up the connection for a query
		/// </summary>
		/// <returns></returns>
		protected bool Open()
		{
			/// create if it doesn't exist already
			if (this.Conn == null) 
			{
				try 
				{
					this.Conn = new SqlConnection(this.ConnectionString);
				}
				catch(Exception e)
				{
					this.SetError( e.Message );
					return false;
				}
			}

			/// check if connection is open - if not open it
			if (this.Conn.State != ConnectionState.Open) 
			{
				try 
				{
					Conn.Open();
				}
				catch(Exception e)
				{
					this.SetError( e.Message );
					return false;
				}
			}

			/// make sure our dataset object exists
			if (this.DataSet == null) 
				this.DataSet = new DataSet();

			return true;
		}

		protected bool Close() 
		{
			if (Conn.State == ConnectionState.Open)
				try 
				{
					Conn.Close();
				}
				catch(Exception e)
				{
					this.SetError( e.Message );
					return false;
				}

			return true;
		}

		/// <summary>
		///  Executes a Select statement and sets teh value of a the dataRowSet.
		/// </summary>
		/// <param name="lcSQL"></param>
		/// <param name="lcCursor"></param>
		/// <returns></returns>
		public int Execute(string SQL, string Cursor)
		{
			if (!this.Open()) 
				return 0;

			SqlDataAdapter DSCommand = new SqlDataAdapter();
			DSCommand.SelectCommand = 
				new SqlCommand(SQL,Conn);


			if ( DataSet.Tables.Contains(Cursor) )
					DataSet.Tables.Remove(Cursor);
	
			try 
			{
				DSCommand.Fill(this.DataSet,Cursor);
			}
			catch(Exception e)
			{
				this.SetError(e.Message);
				this.Close();
				return -1;
			}
			
			this.Close();

			return DataSet.Tables[Cursor].Rows.Count;
		}

		// *** HIGHER LEVEL METHODS (very, very basic)

		public bool Load(string Pk)
		{
			if ( this.Execute("select * from " + this.TableName + " where " + this.PkField + "='" + Pk + "'",this.TableName + "_Record") < 0)
				return false;

			this.DataRow = this.DataSet.Tables[this.TableName + "_Record"].Rows[0];

			return true;
		}


		public bool Load(int Pk) 
		{
			if ( this.Execute("select * from " + this.TableName + " where " + this.PkField + "=" + Pk.ToString(),this.TableName + "_Record") < 0)
				return false;

			this.DataRow = this.DataSet.Tables[this.TableName + "_Record"].Rows[0];

            return true;
		}

		public bool Save() 
		{
			if (!this.Open())	
				return true;

			/// Even though we won't execute this command the data adapter needs this 
			/// command to figure out the key field info in order to properly do updates
			SqlDataAdapter oDSCommand = new SqlDataAdapter("select * from " + this.TableName,this.Conn);

			/// This builds the Update/InsertCommands for the data adapter
			SqlCommandBuilder oCmdBuilder = new SqlCommandBuilder(oDSCommand);

			int lnRows = 0;
			try 
			{
				/// Take the changes in the dataset and sync to the database
				lnRows = oDSCommand.Update(this.DataSet,this.TableName + "_Record");
			}
			catch(Exception ex)
			{
				this.SetError(ex.Message);
				return false;
			}
			finally 
			{
				this.Close();
			}

			if ( lnRows < 1 )
				return false;

			return true;
		}

		
		
		protected void SetError(string lcErrormsg)
		{
			if (lcErrormsg.Length == 0) 
			{
				this.ErrorMsg = "";
				this.Error = false;
				return;
			}

			this.ErrorMsg = lcErrormsg;
			this.Error = true;
		}

	}
}
