using System;

namespace Westwind.Web.Data
{

	/// <summary>
	/// Summary description for IwwData.
	/// </summary>
	public interface IwwWebDataControl
	{
		
		/// <summary>
		/// The object that the control is bound to. This is the base object
		/// and you are still required to set the BindingSource property with
		/// the property to bind to
		/// </summary>
		string BindingSourceObject
		{ 
			get; set;
		}

		string Text
		{  
			get; set;
		}

		/// <summary>
		/// The BindingSource property to bind to set as a string. Uses Reflection to retrieve and set 
		/// content of the object in question
		/// </summary>
		string BindingSourceProperty 
		{
			get; set;
		}
		

		/// <summary>
		/// Field or type that this control is bound to
		/// </summary>
		string BindingProperty
		{
			get; set;
		}

		/// <summary>
		/// A format string used to display the data when binding
		/// </summary>
		string DisplayFormat 
		{
			get; set;
		}

		/// <summary>
		/// The fieldname the user sees when an error occurs.
		/// </summary>
		string UserFieldName
		{
			get;set;
		}


		/// <summary>
		/// Error message that is set if unbinding fails.
		/// </summary>
		string BindingErrorMessage
		{
			get;  set; 
		}

		/// <summary>
		/// Hooks up databinding for the control.
		/// </summary>
		/// <param name="loContainer">Object or Control that contains the BindingSource</param>
		void BindData(System.Web.UI.Page WebForm) ;

		/// <summary>
		/// Unbinds data back into its BindingSource.
		/// </summary>
		/// <param name="loParent">Object that contains the BindingSource</param>
		void UnbindData(System.Web.UI.Page WebForm) ;

	}
}

