using System;
using System.IO;
using System.Text;
using System.Reflection;


namespace Westwind.Web.Data
{

	/// <summary>
	/// Summary description for Class1.
	/// </summary>
	public class wwUtils
	{
	
		/// <summary>
		/// Binding Flags constant to be reused for all Reflection access methods.
		/// </summary>
		public const BindingFlags MemberAccess = 
			BindingFlags.Public | BindingFlags.NonPublic | 
			BindingFlags.Static | BindingFlags.Instance | BindingFlags.IgnoreCase ;
			
		/// <summary>
		/// Determines whether a string is empty (null or zero length)
		/// </summary>
		/// <param name="String">Input string</param>
		/// <returns></returns>
		public static bool Empty(string String) 
		{
			if (String == null || String.Trim().Length == 0)
				return true;

			return false;
		}

		/// <summary>
		/// Replaces and  and Quote characters to HTML safe equivalents.
		/// </summary>
		/// <param name="lcHTML">HTML to convert</param>
		/// <returns>Returns an HTML string of the converted text</returns>
		public static string FixHTMLForDisplay( string lcHTML )
		{
			lcHTML = lcHTML.Replace("<","&lt;");
			lcHTML = lcHTML.Replace(">","&gt;");
			lcHTML = lcHTML.Replace("\"","&quote;");
			return lcHTML;
		}
	
		/// <summary>
		/// Retrieve a dynamic 'non-typelib' property
		/// </summary>
		/// <param name="loObject"></param>
		/// <param name="lcProperty"></param>
		/// <returns></returns>
		public static object GetProperty(object loObject,string lcProperty)
		{
			 return loObject.GetType().GetProperty(lcProperty,wwUtils.MemberAccess).GetValue(loObject,null);
//				loObject.GetType().InvokeMember(lcProperty,BindingFlags.GetProperty,null,loObject,null);
		}

		/// <summary>
		/// Retrieve a dynamic 'non-typelib' field
		/// </summary>
		/// <param name="loObject"></param>
		/// <param name="lcProperty"></param>
		/// <returns></returns>
		public static object GetField(object loObject,string lcProperty)
		{
			return loObject.GetType().GetField(lcProperty,wwUtils.MemberAccess).GetValue(loObject);
			//return loObject.GetType().InvokeMember(lcProperty,BindingFlags.GetField,null,loObject,null);
		}

		/// <summary>
		/// Returns a property or field value using a base object and sub members including . syntax.
		/// For example, you can access: this.oCustomer.oData.Company with (this,"oCustomer.oData.Company")
		/// </summary>
		/// <param name="loParent">Parent object to 'start' parsing from.</param>
		/// <param name="lcProperty">The property to retrieve. Example: 'oBus.oData.Company'</param>
		/// <returns></returns>
		public static object GetPropertyEx(object loParent, string lcProperty) 
		{
			MemberInfo Member = null;

			Type loType = loParent.GetType();

			int lnAt = lcProperty.IndexOf(".");
			if ( lnAt < 0) 
			{
				if (lcProperty == "this" || lcProperty == "me")
					return loParent;

				// *** Get the member
				Member = loType.GetMember(lcProperty,wwUtils.MemberAccess)[0];
				if (Member.MemberType == MemberTypes.Property )
				{
					return ((PropertyInfo) Member).GetValue(loParent,null);

//					return  loType.InvokeMember(lcProperty,
//											System.Reflection.BindingFlags.GetProperty,
//											null,loParent,null);
				}
				else
				{
					return ((FieldInfo) Member).GetValue(loParent);

//					return  loType.InvokeMember(lcProperty,
//							System.Reflection.BindingFlags.GetField,
//							null,loParent,null);
				}
			}

			// *** Walk the . syntax - split into current object (lcMain) and further parsed objects (lcSubs)
			string lcMain = lcProperty.Substring(0,lnAt);
			string lcSubs = lcProperty.Substring(lnAt+1);

			// *** Retrieve the current property
			Member = loType.GetMember(lcMain,wwUtils.MemberAccess)[0];
			
			object loSub;
			if (Member.MemberType == MemberTypes.Property )
			{
				// *** Get its value
			    loSub = ((PropertyInfo) Member).GetValue(loParent,null);

//				loSub = loType.InvokeMember(lcMain,
//					System.Reflection.BindingFlags.GetProperty,
//					null,loParent,null);
			}
			else
			{
				loSub = ( (FieldInfo) Member).GetValue(loParent);

//				loSub = loType.InvokeMember(lcMain,
//					System.Reflection.BindingFlags.GetField,
//					null,loParent,null);
			}

			// *** Recurse further into the sub-properties (lcSubs)
			return wwUtils.GetPropertyEx(loSub,lcSubs);
		}

		public static void SetProperty(object loObject,string lcProperty,object loValue)
		{
			loObject.GetType().GetProperty(lcProperty,wwUtils.MemberAccess).SetValue(loObject,loValue,null);
			//return loObject.GetType().InvokeMember(lcProperty,BindingFlags.SetProperty,null,loObject,loValue);
		}

		public static void SetField(object loObject,string lcProperty,object loValue)
		{
			loObject.GetType().GetField(lcProperty,wwUtils.MemberAccess).SetValue(loObject,loValue);
			//return loObject.GetType().InvokeMember(lcProperty,BindingFlags.SetField,null,loObject,loValue);
		}

		public static object SetPropertyEx(object loParent, string lcProperty,object loValue) 
		{
			Type loType = loParent.GetType();
			MemberInfo Member = null;

			// *** no more .s - we got our final object
			int lnAt = lcProperty.IndexOf(".");
			if ( lnAt < 0) 
			{
				Member = loType.GetMember(lcProperty,wwUtils.MemberAccess)[0];
				if ( Member.MemberType == MemberTypes.Property ) 
				{

					((PropertyInfo) Member).SetValue(loParent,loValue,null);
					//					return  loType.InvokeMember(lcProperty,
					//						System.Reflection.BindingFlags.SetProperty,
					//						null,loParent,loValue);
					return null;
				}
				else 
				{
					((FieldInfo) Member).SetValue(loParent,loValue);
					//					return  loType.InvokeMember(lcProperty,
					//						System.Reflection.BindingFlags.SetField,
					//						null,loParent,loValue);
					return null;				   
				}
			}	
				
			// *** Walk the . syntax
			string lcMain = lcProperty.Substring(0,lnAt);
			string lcSubs = lcProperty.Substring(lnAt+1);
			Member = loType.GetMember(lcMain,wwUtils.MemberAccess)[0];

			object loSub;
			if (Member.MemberType == MemberTypes.Property)
				loSub = ((PropertyInfo) Member).GetValue(loParent,null);
//				loSub = loType.InvokeMember(lcMain,
//					System.Reflection.BindingFlags.GetProperty,
//					null,loParent,null);
			else
				loSub = ((FieldInfo) Member).GetValue(loParent);
//			loSub = loType.InvokeMember(lcMain,
//					System.Reflection.BindingFlags.GetField,
//					null,loParent,null);

			// *** Recurse until we get the lowest ref
			SetPropertyEx(loSub,lcSubs,loValue);
			return null;
		}

		/// <summary>
		/// Wrapper method to call a 'dynamic' (non-typelib) method
		/// on a COM object
		/// </summary>
		/// <param name="loParams"></param>
		/// 1st - Method name, 2nd - 1st parameter, 3rd - 2nd parm etc.
		/// <returns></returns>
		public static object CallMethod(object loObject,string lcMethod, params object[] loParams)
		{
			
			return loObject.GetType().GetMethod(lcMethod,wwUtils.MemberAccess).Invoke(loObject,loParams);

			//return loObject.GetType().InvokeMember(lcMethod,
			//	BindingFlags.InvokeMethod,null,loObject,loParams);
		}

	}
}
