using System;
using System.Web;

using System.Web.UI;
using System.Web.UI.WebControls;
using System.ComponentModel;

using System.Data;
using System.Reflection;
using System.Globalization;


namespace Westwind.Web.Data
{
	/// <summary>
	/// Summary description for wwWebDataControlHelper.
	/// </summary>
	public class wwWebDataHelper
	{

		/// <summary>
		/// Binds all databound controls on a WebForm page by calling their BindData methods.
		/// </summary>
		/// <param name="Container">the parent container (Page)</param>
		public static void FormBindData(Page WebForm) 
		{
			FormBindData((Control) WebForm, WebForm);
		}

		/// <summary>
		/// Binds all databound controls on a WebForm page by calling their BindData methods.
		/// </summary>
		/// <param name="Container">the control to databind</param>
		/// <param name="Form">the master container that contains the control (page)</param>
		static void FormBindData(Control Container, Page WebForm) 
		{
			// *** Drill through each control on the form
			foreach( Control loControl in Container.Controls)
			{
				// ** Recursively call down into any containers
				if (loControl.Controls.Count > 0)
					wwWebDataHelper.FormBindData(loControl, WebForm);

				// ** only work on those that support interface
				if (loControl is IwwWebDataControl ) 
				{
					IwwWebDataControl control = (IwwWebDataControl) loControl;

					try 
					{
						//*** Call the BindData method on the control
						control.GetType().GetMethod("BindData",wwUtils.MemberAccess).Invoke(control,new object[1] { WebForm } );
						//						control.GetType().InvokeMember("BindData",System.Reflection.BindingFlags.InvokeMethod,
						//							null,control, new object[1] { WebForm } );
					}
					catch(Exception) 
					{
						// *** Display Error info
						try 
						{
							///TODO: This must be adjusted for specific types of controls
							control.Text = "** Field binding Error **";
						}
						catch(Exception) {;} 
					}
				}
			}
						
		}

		public static BindingError[] FormUnbindData(Page WebForm) 
		{
			BindingError[] Errors = null;
			FormUnbindData(WebForm,WebForm,ref Errors);
			return Errors;
		}
		
		static BindingError[] FormUnbindData(Control Container, Page WebForm, ref BindingError[] Errors) 
		{
	
			// *** Drill through each of the controls
			foreach( Control loControl in Container.Controls)
			{
				// ** Recursively call down into containers
				if (loControl.Controls.Count > 0)
					FormUnbindData(loControl, WebForm,ref Errors);

				if (loControl is IwwWebDataControl ) 
				{
					IwwWebDataControl control = (IwwWebDataControl) loControl;

					try 
					{
						// *** Call the UnbindData method on the control
						control.GetType().GetMethod("UnBindData",wwUtils.MemberAccess).Invoke(control,new object[1] { WebForm } );

						//						control.GetType().InvokeMember("UnbindData",System.Reflection.BindingFlags.InvokeMethod,
						//							null,control, new object[1] { WebForm } );
					}
					catch(Exception ex) 
					{
						// *** Display Error info
						try 
						{
							BindingError loError = new BindingError();
							if (wwUtils.Empty(control.BindingErrorMessage))  
							{
								if ( control.UserFieldName != "")
									loError.Message = "Invalid format for " + control.UserFieldName;
								else
									loError.Message = "Invalid format for " + loControl.ID.Replace("txt","");
							}
							else 
								loError.Message = control.BindingErrorMessage;


							control.BindingErrorMessage = loError.Message;

							loError.ErrorMsg = ex.Message;
							loError.Source = ex.Source;
							loError.StackTrace = ex.StackTrace;
							loError.ObjectName = loControl.ID;

							if (Errors == null) 
							{	
								Errors = new BindingError[1];
								Errors[0] = loError;
							}
							else
							{
								// *** Resize the array and assign Error
								int lnSize = Errors.GetLength(0);
								Array loTemp =  Array.CreateInstance(typeof(BindingError),lnSize + 1);
								Errors.CopyTo(loTemp,0);
								loTemp.SetValue(loError,lnSize);

								Errors = (BindingError[]) loTemp;
							}
						}
						catch(Exception) {;} // ignore additional exceptions
					}
				}

			}
			return Errors;
		}

		/// <summary>
		/// Formats an Binding Errors collection as a string in HTML format
		/// </summary>
		/// <param name="Errors"></param>
		/// <param name="FormatAsHtml"></param>
		/// <returns></returns>
		public static string BindingErrorsToHtml(BindingError[] Errors)
		{
			return wwWebDataHelper.BindingErrorsToString(Errors).Replace("\r","<br>");
		}

		/// <summary>
		/// Formats an Binding Errors collection as a string.
		/// </summary>
		/// <param name="Errors"></param>
		/// <returns></returns>
		public static string BindingErrorsToString(BindingError[] Errors) 
		{
			// *** Optional Error Parsing
			if (Errors != null) 
			{
				string lcErrors = "";
				for (int x = 0; x < Errors.Length; x++) 
				{
					lcErrors = lcErrors + Errors[x].Message + "<br>";
				}
				if (!wwUtils.Empty(lcErrors))
					return  lcErrors;
			}

			return "";
		}


		/// <summary>
		/// 
		/// </summary>
		/// <param name="WebPage"></param>
		/// <param name="ActiveControl"></param>
		public static void ControlBindData(Page WebPage, 
			IwwWebDataControl ActiveControl) 
		{

			string BindingSourceObject = ActiveControl.BindingSourceObject;
			string BindingSourceProperty = ActiveControl.BindingSourceProperty;
			string BindingProperty = ActiveControl.BindingProperty;

			try 
			{
				if (BindingSourceObject == null || BindingSourceObject.Length == 0 ||
					BindingSourceProperty == null || BindingSourceProperty.Length == 0)  
					return;

				// *** Get a reference to the actual control source object
				object loBindingSource = null;

				// *** Allow this or me to be bound to the page
				if (BindingSourceObject == "this" || BindingSourceObject.ToLower() == "me")
					loBindingSource = WebPage;
				else  
					loBindingSource = wwUtils.GetPropertyEx(WebPage,BindingSourceObject);

				if (loBindingSource == null)
					return;


				// *** Retrieve the control source value
				object loValue;

				if (loBindingSource is System.Data.DataSet) 
				{
					string lcTable = BindingSourceProperty.Substring(0,BindingSourceProperty.IndexOf("."));
					string lcColumn = BindingSourceProperty.Substring(BindingSourceProperty.IndexOf(".")+1);
					DataSet Ds = (DataSet) loBindingSource;
					loValue = Ds.Tables[lcTable].Rows[0][lcColumn];
				}
				else if(loBindingSource is System.Data.DataRow)
				{
					DataRow Dr = (DataRow) loBindingSource;
					loValue = Dr[BindingSourceProperty];
				}
				else if(loBindingSource is System.Data.DataTable)
				{
					DataTable Dt = (DataTable) loBindingSource;
					loValue = Dt.Rows[0][BindingSourceProperty];
				}
				else if(loBindingSource is System.Data.DataView)
				{
					DataView Dv = (DataView) loBindingSource;
					loValue = Dv.Table.Rows[0][BindingSourceProperty];
				}
				else
				{
					loValue = wwUtils.GetPropertyEx(loBindingSource,BindingSourceProperty);
				}

				/// *** Figure out the type of the control we're binding to
				object loBindValue = wwUtils.GetProperty(ActiveControl,BindingProperty);
				string lcBindingSourceType = loBindValue.GetType().Name;

				if (loValue == null)
					if (lcBindingSourceType == "String")
						wwUtils.SetProperty(ActiveControl,BindingProperty,"");
					else if (lcBindingSourceType == "Boolean")
						wwUtils.SetProperty(ActiveControl,BindingProperty,false);
					else
						wwUtils.SetProperty(ActiveControl,BindingProperty,"");
				else 
				{
					///TODO: Fix Formatting here					
					if (lcBindingSourceType == "Boolean") 
						wwUtils.SetProperty(ActiveControl,BindingProperty,loValue);
					else 
					{
						if (wwUtils.Empty(ActiveControl.DisplayFormat))
							wwUtils.SetProperty(ActiveControl,BindingProperty,loValue.ToString());
						else 
							wwUtils.SetProperty(ActiveControl,BindingProperty,String.Format(ActiveControl.DisplayFormat,loValue));

					}
				}
			}
			catch(Exception ex) 
			{
				string lcException = ex.Message;
				throw(new Exception("Can't bind " + ((Control) ActiveControl).ID  + " to " + 
					BindingSourceObject + "." + 
					BindingSourceProperty));
			}
		}

		/// <summary>
		/// Unbinds control properties back into the control source
		/// </summary>
		/// <param name="WebPage">The WebForm Page object</param>
		/// <param name="ActiveControl">The control to unbind</param>
		public static void ControlUnbindData(Page WebPage, 
			IwwWebDataControl ActiveControl) 
		{
			string BindingSourceObject = ActiveControl.BindingSourceObject;
			string BindingSourceProperty = ActiveControl.BindingSourceProperty;
			string BindingProperty = ActiveControl.BindingProperty;

			if (BindingSourceObject == null || BindingSourceObject.Length == 0 ||
				BindingSourceProperty == null || BindingSourceProperty.Length == 0)  
				return;

			object loBindingSource = null;
			if (BindingSourceObject == "this" || BindingSourceObject.ToLower() == "me")
				loBindingSource = WebPage;
			else  
				loBindingSource = wwUtils.GetPropertyEx(WebPage,BindingSourceObject);

			if (loBindingSource == null) 
				throw(new Exception("Invalid BindingSource"));

			// Retrieve the new value from the control
			object loValue = wwUtils.GetPropertyEx(ActiveControl,BindingProperty);

			// Try to retrieve the type of the BindingSourceProperty
			string lcBindingSourceType;
			string lcDataColumn = null;
			string lcDataTable = null;

			if (loBindingSource is System.Data.DataSet) 
			{
				// *** Split out the datatable and column names
				int lnAt = BindingSourceProperty.IndexOf(".");
				lcDataTable = BindingSourceProperty.Substring(0,lnAt);
				lcDataColumn = BindingSourceProperty.Substring(lnAt+1);
				DataSet Ds = (DataSet) loBindingSource;
				lcBindingSourceType = Ds.Tables[lcDataTable].Columns[lcDataColumn].DataType.Name;
			}
			else if(loBindingSource is System.Data.DataRow)
			{
				DataRow Dr = (DataRow) loBindingSource;
				lcBindingSourceType = Dr.Table.Columns[BindingSourceProperty].DataType.Name;
			}
			else if (loBindingSource is System.Data.DataTable) 
			{
				DataTable dt = (DataTable) loBindingSource;
				lcBindingSourceType = dt.Columns[BindingSourceProperty].DataType.Name;
			}
			else 
			{
				// *** It's an object property or field - get it
				MemberInfo[] loInfo = loBindingSource.GetType().GetMember(BindingSourceProperty,wwUtils.MemberAccess);
				if (loInfo[0].MemberType == MemberTypes.Field) 
				{
					FieldInfo loField = (FieldInfo) loInfo[0];
					lcBindingSourceType = loField.FieldType.Name;
				}
				else
				{
					PropertyInfo loField = (PropertyInfo) loInfo[0];
					lcBindingSourceType = loField.PropertyType.Name;
				}
			}

			/// Retrieve the value
			object loAssignedValue;

			if ( lcBindingSourceType == "String")
				loAssignedValue = loValue;
			else if (lcBindingSourceType  == "Int16")  
				loAssignedValue = Int16.Parse( (string) loValue, NumberStyles.Integer ); // Convert.ToInt16(loValue);				
			else if (lcBindingSourceType  == "Int32")  
				loAssignedValue = Int32.Parse( (string) loValue, NumberStyles.Integer ); // Convert.ToInt32(loValue);				
			else if (lcBindingSourceType  == "Int64")  
				loAssignedValue = Int32.Parse ( (string) loValue, NumberStyles.Integer);				
			else if (lcBindingSourceType  == "Byte")  
				loAssignedValue = Convert.ToByte(loValue);				
			else if (lcBindingSourceType  == "Decimal")  
				loAssignedValue = Decimal.Parse( (string) loValue,NumberStyles.Any);
			else if (lcBindingSourceType  == "Double")  
				loAssignedValue = Double.Parse( (string) loValue,NumberStyles.Any);				
			else if (lcBindingSourceType  == "Boolean") 
			{
				loAssignedValue = loValue;
			}
			else if (lcBindingSourceType  == "DateTime")  
				loAssignedValue = Convert.ToDateTime(loValue);				
			else  // Not HANDLED!!!
				throw(new Exception("Field Type not Handled by Data unbinding"));

			/// Write the value back to the underlying object/data item
			if (loBindingSource is System.Data.DataSet) 
			{
				DataSet Ds = (DataSet) loBindingSource;
				Ds.Tables[lcDataTable].Rows[0][lcDataColumn] = loAssignedValue;
			}
			else if(loBindingSource is System.Data.DataRow) 
			{
				DataRow Dr = (DataRow) loBindingSource;
				Dr[BindingSourceProperty] = loAssignedValue;
			}	
			else if(loBindingSource is System.Data.DataTable) 
			{
				DataTable dt = (DataTable) loBindingSource;
				dt.Rows[0][BindingSourceProperty] = loAssignedValue;
			}
			else if(loBindingSource is System.Data.DataView) 
			{
				DataView dv = (DataView) loBindingSource;
				dv[0][BindingSourceProperty] = loAssignedValue;
			}
			else
				wwUtils.SetPropertyEx(loBindingSource,BindingSourceProperty,loAssignedValue);
	
			
			// *** Clear the error message - no error
			ActiveControl.BindingErrorMessage = ""; 
		}

		

	}



	/// <summary>
	/// Error object used to return error information during databinding.
	/// </summary>
	public class BindingError 
	{
		/// <summary>
		/// Name of the object the error occurred on
		/// </summary>
		public string ObjectName
		{
			get { return this.cObjectName; }
			set { this.cObjectName = value; }
		}
		string cObjectName;

		/// <summary>
		/// The error message raised
		/// </summary>
		public string Message
		{
			get { return this.cMessage; }
			set { this.cMessage = value; }
		}
		string cMessage;

		/// <summary>
		/// The raw Exception error message
		/// </summary>
		public string ErrorMsg
		{
			get { return this.cErrorMsg; }
			set { this.cErrorMsg = value; }
		}
		string cErrorMsg;

		/// <summary>
		/// The code that caused the error
		/// </summary>
		public string Source
		{
			get { return this.cSource; }
			set { this.cSource = value; }
		}
		string cSource;
		
		/// <summary>
		/// The call stack that led up to the error
		/// </summary>
		public string StackTrace
		{
			get { return this.cStackTrace; }
			set { this.cStackTrace = value; }
		}
		string cStackTrace;
	}

}
